from juntagrico.config import Config

from juntagrico_assignment_request.dao.assignmentrequestdao import AssignmentRequestDao

'''
Server generated Emails
'''


def get_approver_emails(assignment_request):
    receivers = []
    if assignment_request.approver:
        # Notify approver if defined
        receivers.append(assignment_request.approver.email)
    else:
        # Otherwise inform all that should be notified on unapproved assignments
        for notified in AssignmentRequestDao.all_notified_on_unapproved_assignments():
            receivers.append(notified.email)
    if not receivers:
        # Nobody? Send it to the info mail then
        receivers.append(Config.info_email())
    return receivers



# Generated by Django 3.2.2 on 2021-11-14 17:39

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('juntagrico', '0034_auto_20210415_2041'),
        ('juntagrico_assignment_request', '0002_fix_naming_and_relation'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignmentrequest',
            name='activityarea',
            field=models.ForeignKey(blank=True, help_text='Was am besten passt. Ansonsten leer lassen', null=True, on_delete=django.db.models.deletion.SET_NULL, to='juntagrico.activityarea', verbose_name='Tätigkeitsbereich'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='amount',
            field=models.PositiveIntegerField(default=1, help_text='Wieviele Arbeitseinsätze?', validators=[django.core.validators.MinValueValidator(1)], verbose_name='Wert'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='approver',
            field=models.ForeignKey(blank=True, help_text='An wen soll die Anfrage gesendet werden oder wer hat dich aufgefordert, die Anfrage zu senden?', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='Referenz', to='juntagrico.member', verbose_name='Abgesprochen mit'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='description',
            field=models.TextField(default='', help_text='Kurze Beschreibung was du gemacht hat', max_length=1000, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='location',
            field=models.CharField(blank=True, default='', help_text='Optional', max_length=100, verbose_name='Ort'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='member',
            field=models.ForeignKey(help_text='Beantragt von', on_delete=django.db.models.deletion.CASCADE, to='juntagrico.member', verbose_name='Mitglied'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='response',
            field=models.TextField(blank=True, help_text='Rückmeldung an Mitglied. Kann leer bleiben.', null=True, verbose_name='Antwort'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='response_date',
            field=models.DateField(blank=True, help_text='Typischerweise heute', null=True, verbose_name='Beantwortet am'),
        ),
        migrations.AlterField(
            model_name='assignmentrequest',
            name='status',
            field=models.CharField(choices=[('RE', 'Beantragt'), ('NO', 'Abgelehnt'), ('CO', 'Bestätigt')], default='RE', help_text='Hier "Bestätigt" auswählen, um die Anfrage anzunehmen', max_length=2),
        ),
    ]



import os
from setuptools import find_packages, setup

import juntagrico_assignment_request

ROOT_DIR = os.path.dirname(__file__)
SOURCE_DIR = os.path.join(ROOT_DIR)

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


def get_requirements(requirements_file):
    with open(requirements_file) as f:
        required = [line.split('#')[0] for line in f.read().splitlines()]
    return required


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=juntagrico_assignment_request.name,
    version=juntagrico_assignment_request.version,
    packages=find_packages(),
    include_package_data=True,
    license='LPGLv3',  # example license
    description='juntagrico_assignment_request',
    long_description=README,
    url='https://github.com/juntagrico/juntagrico-assignment-request',
    author='juntagrico',
    author_email='info@juntagrico.org',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8 ',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    install_requires=get_requirements(os.path.join(ROOT_DIR, 'requirements.txt')),
)



from django.forms import ModelForm
from django.utils.translation import gettext_lazy as _
from django.utils.translation import gettext
from django.urls import reverse

from juntagrico_assignment_request.dao.assignmentrequestdao import AssignmentRequestDao
from juntagrico_assignment_request.models import AssignmentRequest

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Submit, HTML
from crispy_forms.bootstrap import FormActions


class AssignmentRequestForm(ModelForm):
    class Meta:
        model = AssignmentRequest
        fields = ('job_time', 'amount', 'duration', 'approver',
                  'activityarea', 'location', 'description')
        labels = {
            "amount": _("Anzahl Einsätze"),
            "approver": _("Abgesprochen mit"),
            "activityarea": _("Tätigkeitsbereich"),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['approver'].queryset = AssignmentRequestDao.all_approvers()
        self.fields['amount'].widget.attrs['min'] = 1
        self.fields['job_time'].widget.format = '%d.%m.%Y %H:%M'
        self.fields['job_time'].widget.attrs['placeholder'] = _('TT.MM.JJJJ HH:MM')
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.layout = Layout(
            'job_time', 'amount', 'duration', 'approver',
            'activityarea', 'location', 'description',
            FormActions(
                Submit('submit', _('Absenden'), css_class='btn-success'),
            )
        )


class AssignmentResponseForm(ModelForm):
    class Meta:
        model = AssignmentRequest
        fields = ('amount', 'duration', 'activityarea', 'location', 'response')
        labels = {
            "amount": _("Anzahl Einsätze"),
            "activityarea": _("Tätigkeitsbereich"),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['amount'].widget.attrs['min'] = 1
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.layout = Layout(
            'amount', 'duration', 'activityarea', 'location', 'response',
            FormActions(
                Submit('confirm', _('Bestätigen'), css_class='btn-success'),
                Submit('reject', _('Ablehnen'), css_class='btn-danger'),
                Submit('submit', _('Nur Antwort senden'), css_class='btn-warning'),
                HTML('<a href="' + reverse('ar-list-assignment-requests') + '" class="btn">' + gettext("Abbrechen") + '</a>'),
            )
        )



from django.utils.translation import gettext as _
from django.template.loader import get_template

from juntagrico.mailer import EmailSender, base_dict
from juntagrico.config import Config

from juntagrico_assignment_request.config import AssignmentRequestConfig


def request_handled(assignment_request):
    d = base_dict({'assignment_request': assignment_request})

    if assignment_request.is_confirmed():
        subject = _('{} bestätigt').format(Config.vocabulary('assignment'))
        content = get_template(AssignmentRequestConfig.emails('confirmed_assignment_request_mail')).render(d)
    elif assignment_request.is_rejected():
        subject = _('{} nicht bestätigt').format(Config.vocabulary('assignment'))
        content = get_template(AssignmentRequestConfig.emails('rejected_assignment_request_mail')).render(d)
    else:
        subject = _('Rückfrage zu deinem/r {}').format(Config.vocabulary('assignment'))
        content = get_template(AssignmentRequestConfig.emails('responded_assignment_request_mail')).render(d)

    EmailSender.get_sender(Config.organisation_name() + ' - ' + subject, content,
                           reply_to=[assignment_request.approver.email]).send_to(assignment_request.member.email)



from datetime import date

from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Q
from django.shortcuts import render, redirect, get_object_or_404
from juntagrico.view_decorators import highlighted_menu

from juntagrico_assignment_request.dao.assignmentrequestdao import AssignmentRequestDao
from juntagrico_assignment_request.forms import AssignmentRequestForm, AssignmentResponseForm
from juntagrico_assignment_request.mailer import membernotification, adminnotification
from juntagrico_assignment_request.models import AssignmentRequest


@login_required
@highlighted_menu('request_assignment')
def request_assignment(request, sent=False):
    """
    Request an assignment
    """

    member = request.user.member
    assignment_request_form = AssignmentRequestForm(request.POST or None)
    if request.method == 'POST' and assignment_request_form.is_valid():
        # Create request
        assignment_request = assignment_request_form.instance
        assignment_request.member = member
        assignment_request.save()
        adminnotification.request_created(assignment_request)
        return redirect('ar-assignment-requested')

    renderdict = {
        'assignment_requests': AssignmentRequestDao.current_requests_by_member(member),
        'form': assignment_request_form,
        'sent': sent
    }
    return render(request, "assignment_request/request_assignment.html", renderdict)


@login_required
def delete_request_assignment(request, request_id):
    """
    Delete an assignment request
    """

    assignment_request = get_object_or_404(AssignmentRequest, id=request_id)
    if assignment_request.member == request.user.member\
            and not assignment_request.assignment:
        assignment_request.delete()
    return redirect('ar-request-assignment')


@login_required
@highlighted_menu('request_assignment')
def edit_request_assignment(request, request_id):
    """
    Edit and assignment request
    """

    member = request.user.member
    assignment_request = get_object_or_404(AssignmentRequest, id=request_id, member=member)
    assignment_request_form = AssignmentRequestForm(request.POST or None, instance=assignment_request)
    if request.method == 'POST' and assignment_request_form.is_valid():
        # edit request
        assignment_request = assignment_request_form.instance
        assignment_request.status = AssignmentRequest.REQUESTED
        assignment_request.save()
        adminnotification.request_changed(assignment_request)
        return redirect('ar-assignment-requested')

    renderdict = {
        'form': assignment_request_form,
    }
    return render(request, "assignment_request/edit_assignment_request.html", renderdict)


def filter_approver(user):
    if user.has_perm('juntagrico_assignment_request.notified_on_unapproved_assignments'):
        return Q(approver=user.member) | Q(approver__isnull=True)
    return Q(approver=user.member)


@permission_required('juntagrico_assignment_request.can_confirm_assignments')
def list_assignment_requests(request):
    """
    List assignment requests
    """
    ar = AssignmentRequest.objects.filter(status=AssignmentRequest.REQUESTED).filter(filter_approver(request.user))
    renderdict = {
        'assignment_requests': ar,
    }
    return render(request, "assignment_request/list_assignment_requests.html", renderdict)


@permission_required('juntagrico_assignment_request.can_confirm_assignments')
def list_archive(request):
    ar = AssignmentRequest.objects.exclude(status=AssignmentRequest.REQUESTED).filter(filter_approver(request.user))
    renderdict = {
        'assignment_requests': ar,
        'archive': True
    }
    return render(request, "assignment_request/list_assignment_requests.html", renderdict)


@permission_required('juntagrico_assignment_request.can_confirm_assignments')
def respond_assignment_request(request, request_id):
    """
    Confirm or reject an assignment request
    """

    assignment_request = get_object_or_404(AssignmentRequest, id=request_id)
    assignment_response_form = AssignmentResponseForm(request.POST or None, instance=assignment_request)
    if request.method == 'POST' and assignment_response_form.is_valid():
        assignment_request = assignment_response_form.instance
        assignment_request.response_date = date.today()
        if 'confirm' in request.POST:
            assignment_request.status = AssignmentRequest.CONFIRMED
        elif 'reject' in request.POST:
            assignment_request.status = AssignmentRequest.REJECTED
        adminnotification.request_handled_by_other_approver(assignment_request, request.user.member)
        assignment_request.approver = request.user.member
        assignment_request.save()
        membernotification.request_handled(assignment_request)
        return redirect('ar-list-assignment-requests')

    renderdict = {
        'assignment_request': assignment_request,
        'form': assignment_response_form,
    }
    return render(request, "assignment_request/respond_assignment_request.html", renderdict)


@permission_required('juntagrico_assignment_request.can_confirm_assignments')
def confirm_assignment_request(request, request_id):
    """
    Confirm an assignment request directly
    """
    assignment_request = get_object_or_404(AssignmentRequest, id=request_id)
    if not assignment_request.is_confirmed():
        assignment_request.response_date = date.today()
        assignment_request.response = ''
        assignment_request.status = AssignmentRequest.CONFIRMED
        adminnotification.request_handled_by_other_approver(assignment_request, request.user.member)
        # overwrite approver in any case to actual approver
        assignment_request.approver = request.user.member
        assignment_request.save()
        membernotification.request_handled(assignment_request)
    return redirect('ar-list-assignment-requests')
